; ************
; *Preamp S1 *
; ************
;
; Purpose:
; Control software for Preamp S1-project
; Reads keys/buttons, controls input selection and power supply
;
; History of development:
; 30.08.2010: Start & 1st release working
; 31.08.2010: Comments reworked
;
;
; Used Processor: PIC 16F690
;
; Pins & Ports:
; RA0: key CD
; RA1: key DVD
; RA2: key PWR (on/off)
; RA3: key TV
; RA4: key AUX
; RA5: key Rec
; RB7: Power-LED
; RC0: relais for CD
; RC1: relais for DVD
; RC2: relais for TV
; RC3: relais for AUX
; RC4: relais for REC
; RC5: output for power supply (1=on)
; RC6: relais for Mute (1= not muted)
;
; Assuming Fosc=INTOSC 500kHz
	
	LIST   P=PIC16F690

	#include <p16f690.inc> 

errorlevel  -302		;No bank select warnings

	__config (_INTOSCIO & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOD_OFF & _IESO_OFF & _FCMEN_OFF) 
;----------------------------------------------------------------
;variables
	cblock 0x20
	d1				;Delay-Variable 1	
	d2				;Delay-Variable 2
	outstat			;Status-Byte for Output (Relais, Power)
	eprom_adr		;eeprom Adress temp
	eprom_dat		;eeprom data temp
	
	endc
;----------------------------------------------------------------
;definitions

#define	CD		PORTA,0	;Key for CD
#define	DVD		PORTA,1 ; ...
#define	PWR		PORTA,2
#define	TV		PORTA,3
#define	AUX		PORTA,4
#define	REC		PORTA,5
#define	LED		PORTB,7 ; Output PWR-LED

#define Outport	PORTC
;----------------------------------------------------------------
;constants

CD_out		equ 0	;used for bitsetting of outport
DVD_out		equ 1
TV_out		equ 2
AUX_out		equ 3
REC_out		equ 4
PWR_out		equ 5
MUTE_out	equ 6

; **********************************************************************
; **********************************************************************
	org 	0 				;Reset vector
	goto 	INIT
	ORG     0x004           ; interrupt vector location
							;no interrupts...
							
;------------------------------------------------------------------
;initializes Processor
;------------------------------------------------------------------
INIT

	
	bsf 	STATUS,RP0 	;select Register Bank 1 
	
	bcf		OSCCON,IRCF2	;select 500kHz
	bsf		OSCCON,IRCF1
	bsf		OSCCON,IRCF0
	
	;confugure input/output

	movlw	0x80		
	movwf	TRISC		;all but RC7 output
	movlw	0x7f
	movwf	TRISB		;all but RB7 input
	movlw	0xff
	movwf	TRISA		;all input (keys)

	clrf	WPUA		;no weak pullup Port A
	
	bcf 	STATUS,RP0 	;back to Register Bank 0 
	bsf 	STATUS,RP1	;to Bank 2

	;configure analog I/O
	clrf	WPUB		;no weak pullup Port B
	
	clrf 	ANSEL		;all digital I/O
	clrf 	ANSELH		;all digital I/O

	bcf		INTCON,GIE	;no global interrupts
	bsf		INTCON,INTE	;external INT Pin for interrupt used for sleep
	bcf		INTCON,INTF	;clear INT flag
	
	bcf 	STATUS, RP1 ;back to Bank 0

;#############################################################################
;here to start main routine###################################################
;#############################################################################
startup
	;set reasonable values to variables
	bcf		LED					;LED off
	clrf	Outport				;all outputs off
	

	bsf		Outport,PWR_out		;Set Power supply on
	bsf		LED					;Power-LED on
	call 	Delay1s				;wait to stabilize
	;load preferences from eeprom
	movlw	0x00				;Set Adress for read of config
	movwf	eprom_adr			;save to adress
	call 	eprom_read			;load old config
	movwf	outstat				;save to current status
	bsf		outstat,PWR_out		;set bit for power supply
	bcf		outstat,MUTE_out	;clear MUTE-Bit (so it is muted)
	movf	outstat,w
	movwf	Outport				;set all to output
	call	Delay100ms
	
	bsf		outstat,MUTE_out	;set Mute-Bit (set=not muted)
	movf	outstat,w
	movwf	Outport	
	
	
mainloop

	btfss	CD					;CD-key pressed?
	goto	ml1					;no, next check
	movf	outstat,w			;get current status
	andlw	b'11100000'			;clear all inputs /relais
	movwf	outstat				;save
	movwf	Outport			
	call	Delay100ms			
	bsf		outstat,CD_out		;set CD-Relais
	movf	outstat,w	
	movwf	Outport
	btfsc	CD					;wait for key-release
	goto	$-1
	call	Delay100ms			;used for key debouncing....
	
ml1
	btfss	DVD					;DVD-key pressed?
	goto	ml2					;no, next check
	movf	outstat,w			;get current status
	andlw	b'11100000'			;clear all inputs
	movwf	outstat				;save
	movwf	Outport			
	call	Delay100ms
	bsf		outstat,DVD_out		;set CD-Relais
	movf	outstat,w	
	movwf	Outport
	btfsc	DVD					;wait for key-release
	goto	$-1
	call	Delay100ms			;used for key debouncing....
	
ml2	
	btfss	TV					;TV-key pressed?
	goto	ml3					;no, next check
	movf	outstat,w			;get current status
	andlw	b'11100000'			;clear all inputs
	movwf	outstat				;save
	movwf	Outport			
	call	Delay100ms
	bsf		outstat,TV_out		;set CD-Relais
	movf	outstat,w	
	movwf	Outport
	btfsc	TV					;wait for key-release
	goto	$-1
	call	Delay100ms			;used for key debouncing....
	
ml3	
	btfss	AUX					;AUX-key pressed?
	goto	ml4					;no, next check
	movf	outstat,w			;get current status
	andlw	b'11100000'			;clear all inputs
	movwf	outstat				;save
	movwf	Outport			
	call	Delay100ms
	bsf		outstat,AUX_out		;set CD-Relais
	movf	outstat,w	
	movwf	Outport
	btfsc	AUX					;wait for key-release
	goto	$-1
	call	Delay100ms			;used for key debouncing....
	
ml4
	btfss	REC					;REC-key pressed?
	goto	ml5					;no, next check
	movf	outstat,w			;get current status
	andlw	b'11100000'			;clear all inputs
	movwf	outstat				;save
	movwf	Outport			
	call	Delay100ms
	bsf		outstat,REC_out		;set CD-Relais
	movf	outstat,w	
	movwf	Outport
	btfsc	REC					;wait for key-release
	goto	$-1
	call	Delay100ms			;used for key debouncing....
	
ml5
	btfsc	PWR					;PWR-Button pressed
	goto	DoSleep				;yes, then end
	
	goto	mainloop

;-----------------------------------------------------------------
; Do Sleep
; Sets PIC to sleep after saving prefs
;-----------------------------------------------------------------
DoSleep
	clrf	eprom_adr			;Adress to save to 
	movf	outstat,w
	andlw	b'00011111'			;load status and mask PWR&MUTE
	movwf	eprom_dat
	call	eprom_write			;save to eprom
	bcf		Outport,MUTE_out	;Mute output
	call	Delay100ms
	call	Delay100ms
	clrf	Outport				;all Relais and power supply off
	call	Delay100ms
	call	Delay100ms
	call	Delay100ms
	bcf		LED
	btfsc	PWR					;wait for key release
	goto	$-1
	call	Delay1s
	bcf		INTCON,INTF			;clear INT Flag
	btfsc	INTCON,INTF			;really clear?
	goto	$-2
	
	sleep						;now goto sleep
								;we sleep until PWR is pressed (INT-Pin)
	
	nop
	bcf		INTCON,INTF			;clear INT Flag
	goto	startup				;and go on



;---------------------------------------------------------------
;EEPROM handling routines
;---------------------------------------------------------------
;eprom read: Address in eprom_adr, return value in w
;---------------------------------------------------------------
eprom_read
	bcf		PIR2, EEIF		;clear interrupt flag (necessary?)
	movf	eprom_adr,w		;Set adress of read
	banksel	EEADR
	movwf	EEADR
	
	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to read data mem
	bsf		EECON1,RD		;init read
	banksel	EEDATA
	movf	EEDATA,w
	bcf		STATUS,RP0		;back to bank 0 (our standard)
	bcf		STATUS,RP1
	return
;-------------------------------------------------------------------------
;eprom write Address in eprom_adr, data in eprom_dat returns when finished
;-------------------------------------------------------------------------
eprom_write
	bcf		PIR2, EEIF		;clear interrupt (to be sure)
	movf	eprom_adr,w		;load Adress
	banksel	EEADR
	movwf	EEADR
	banksel	0x00
	movf	eprom_dat,w	;load value
	banksel	EEDATA
	movwf	EEDATA
	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to write data mem
	bsf		EECON1,WREN		;enable write
	
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
	movlw	0x55			;see datasheet, required sequence
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf		EECON1,WR		;set write bit to begin write
	
	banksel	0x00			;back to Bank0 (our default)
	btfss	PIR2,EEIF		;wait for flag to be set (polling)
	goto	$-1
	bcf		PIR2,EEIF		;clear it, write complete

	banksel	EECON1
	bcf		EECON1,WREN		;clear write enable
	banksel	0x00

	return

;--------------------------------------------------------------------------
; Delay routines

; Delay = 0.001 seconds
Delay1ms
			;121 cycles
	movlw	0x28
	movwf	d1
Delay1ms_0
	decfsz	d1, f
	goto	Delay1ms_0
			;4 cycles (including call)
	return

; Delay = 0.1 seconds
Delay100ms
			;12493 cycles
	movlw	0xC2
	movwf	d1
	movlw	0x0A
	movwf	d2
Delay100ms_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay100ms_0
			;3 cycles
	goto	$+1
	nop
			;4 cycles (including call)
	return

; Delay = 1 seconds
Delay1s
			;124993 cycles
	movlw	0xA6
	movwf	d1
	movlw	0x62
	movwf	d2
Delay1s_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay1s_0
			;3 cycles
	goto	$+1
	nop
			;4 cycles (including call)
	return

;end of program
	end
